@@ -21,6 +21,7 @@ from solana.rpc.types import MemcmpOpts
 
 from .accountinfo import AccountInfo
 from .addressableaccount import AddressableAccount
+from .cache import Cache, RootBankCache
 from .context import Context
 from .encoding import encode_key
 from .group import Group
@@ -89,7 +90,7 @@ class Account(AddressableAccount):
                  info: str, shared_quote: AccountSlot,
                  in_margin_basket: typing.Sequence[bool],
                  slot_indices: typing.Sequence[bool],
-                 slots: typing.Sequence[AccountSlot],
+                 base_slots: typing.Sequence[AccountSlot],
                  msrm_amount: Decimal, being_liquidated: bool, is_bankrupt: bool) -> None:
         super().__init__(account_info)
         self.version: Version = version
@@ -102,7 +103,7 @@ class Account(AddressableAccount):
         self.shared_quote: AccountSlot = shared_quote
         self.in_margin_basket: typing.Sequence[bool] = in_margin_basket
         self.slot_indices: typing.Sequence[bool] = slot_indices
-        self.slots: typing.Sequence[AccountSlot] = slots
+        self.base_slots: typing.Sequence[AccountSlot] = base_slots
         self.msrm_amount: Decimal = msrm_amount
         self.being_liquidated: bool = being_liquidated
         self.is_bankrupt: bool = is_bankrupt
@@ -114,13 +115,17 @@ class Account(AddressableAccount):
             raise Exception(f"Shared quote does not have a token: {self.shared_quote}")
         return Token.ensure(token_info.token)
 
+    @property
+    def slots(self) -> typing.Sequence[AccountSlot]:
+        return [*[slot for slot in self.base_slots], self.shared_quote]
+
     @property
     def slots_by_index(self) -> typing.Sequence[typing.Optional[AccountSlot]]:
         mapped_items: typing.List[typing.Optional[AccountSlot]] = []
         slot_counter = 0
         for available in self.slot_indices:
             if available:
-                mapped_items += [self.slots[slot_counter]]
+                mapped_items += [self.base_slots[slot_counter]]
                 slot_counter += 1
             else:
                 mapped_items += [None]
@@ -154,7 +159,7 @@ class Account(AddressableAccount):
 
     @property
     def spot_open_orders(self) -> typing.Sequence[PublicKey]:
-        return [slot.spot_open_orders for slot in self.slots if slot.spot_open_orders is not None]
+        return [slot.spot_open_orders for slot in self.base_slots if slot.spot_open_orders is not None]
 
     @property
     def spot_open_orders_by_index(self) -> typing.Sequence[typing.Optional[PublicKey]]:
@@ -162,14 +167,14 @@ class Account(AddressableAccount):
 
     @property
     def perp_accounts(self) -> typing.Sequence[PerpAccount]:
-        return [slot.perp_account for slot in self.slots if slot.perp_account is not None]
+        return [slot.perp_account for slot in self.base_slots if slot.perp_account is not None]
 
     @property
     def perp_accounts_by_index(self) -> typing.Sequence[typing.Optional[PerpAccount]]:
         return [slot.perp_account if slot is not None else None for slot in self.slots_by_index]
 
     @staticmethod
-    def from_layout(layout: typing.Any, account_info: AccountInfo, version: Version, group: Group) -> "Account":
+    def from_layout(layout: typing.Any, account_info: AccountInfo, version: Version, group: Group, cache: Cache) -> "Account":
         meta_data = Metadata.from_layout(layout.meta_data)
         owner: PublicKey = layout.owner
         info: str = layout.info
@@ -201,9 +206,13 @@ class Account(AddressableAccount):
                 intrinsic_borrow: Decimal = Decimal(0)
                 if token_info is not None:
                     raw_deposit = layout.deposits[index]
-                    intrinsic_deposit = token_info.root_bank.deposit_index * raw_deposit
+                    root_bank_cache: typing.Optional[RootBankCache] = token_info.root_bank_cache_from_cache(
+                        cache, index)
+                    if root_bank_cache is None:
+                        raise Exception(f"No root bank cache found for token {token_info} at index {index}")
+                    intrinsic_deposit = root_bank_cache.deposit_index * raw_deposit
                     raw_borrow = layout.borrows[index]
-                    intrinsic_borrow = token_info.root_bank.borrow_index * raw_borrow
+                    intrinsic_borrow = root_bank_cache.borrow_index * raw_borrow
 
                 deposit = InstrumentValue(instrument, instrument.shift_to_decimals(intrinsic_deposit))
                 borrow = InstrumentValue(instrument, instrument.shift_to_decimals(intrinsic_borrow))
@@ -227,11 +236,16 @@ class Account(AddressableAccount):
             else:
                 active_in_basket += [False]
 
-        raw_quote_deposit: Decimal = layout.deposits[-1]
-        intrinsic_quote_deposit = quote_token_info.root_bank.deposit_index * raw_quote_deposit
+        quote_index: int = len(layout.deposits) - 1
+        raw_quote_deposit: Decimal = layout.deposits[quote_index]
+        quote_root_bank_cache: typing.Optional[RootBankCache] = quote_token_info.root_bank_cache_from_cache(
+            cache, quote_index)
+        if quote_root_bank_cache is None:
+            raise Exception(f"No root bank cache found for quote token {quote_token_info} at index {index}")
+        intrinsic_quote_deposit = quote_root_bank_cache.deposit_index * raw_quote_deposit
         quote_deposit = InstrumentValue(quote_token, quote_token.shift_to_decimals(intrinsic_quote_deposit))
-        raw_quote_borrow: Decimal = layout.borrows[-1]
-        intrinsic_quote_borrow = quote_token_info.root_bank.borrow_index * raw_quote_borrow
+        raw_quote_borrow: Decimal = layout.borrows[quote_index]
+        intrinsic_quote_borrow = quote_root_bank_cache.borrow_index * raw_quote_borrow
         quote_borrow = InstrumentValue(quote_token, quote_token.shift_to_decimals(intrinsic_quote_borrow))
         quote: AccountSlot = AccountSlot(len(layout.deposits) - 1, quote_token_info.token, quote_token_info,
                                          quote_token_info, raw_quote_deposit, quote_deposit, raw_quote_borrow,
@@ -244,21 +258,22 @@ class Account(AddressableAccount):
         return Account(account_info, version, meta_data, group.name, group.address, owner, info, quote, in_margin_basket, active_in_basket, slots, msrm_amount, being_liquidated, is_bankrupt)
 
     @staticmethod
-    def parse(account_info: AccountInfo, group: Group) -> "Account":
+    def parse(account_info: AccountInfo, group: Group, cache: Cache) -> "Account":
         data = account_info.data
         if len(data) != layouts.MANGO_ACCOUNT.sizeof():
             raise Exception(
                 f"Account data length ({len(data)}) does not match expected size ({layouts.MANGO_ACCOUNT.sizeof()})")
 
         layout = layouts.MANGO_ACCOUNT.parse(data)
-        return Account.from_layout(layout, account_info, Version.V3, group)
+        return Account.from_layout(layout, account_info, Version.V3, group, cache)
 
     @staticmethod
     def load(context: Context, address: PublicKey, group: Group) -> "Account":
         account_info = AccountInfo.load(context, address)
         if account_info is None:
             raise Exception(f"Account account not found at address '{address}'")
-        return Account.parse(account_info, group)
+        cache: Cache = group.fetch_cache(context)
+        return Account.parse(account_info, group, cache)
 
     @staticmethod
     def load_all(context: Context, group: Group) -> typing.Sequence["Account"]:
@@ -274,11 +289,12 @@ class Account(AddressableAccount):
 
         results = context.client.get_program_accounts(
             context.mango_program_address, memcmp_opts=filters, data_size=layouts.MANGO_ACCOUNT.sizeof())
-        accounts = []
+        cache: Cache = group.fetch_cache(context)
+        accounts: typing.List[Account] = []
         for account_data in results:
             address = PublicKey(account_data["pubkey"])
             account_info = AccountInfo._from_response_values(account_data["account"], address)
-            account = Account.parse(account_info, group)
+            account = Account.parse(account_info, group, cache)
             accounts += [account]
         return accounts
 
@@ -301,11 +317,12 @@ class Account(AddressableAccount):
 
         results = context.client.get_program_accounts(
             context.mango_program_address, memcmp_opts=filters, data_size=layouts.MANGO_ACCOUNT.sizeof())
-        accounts = []
+        cache: Cache = group.fetch_cache(context)
+        accounts: typing.List[Account] = []
         for account_data in results:
             address = PublicKey(account_data["pubkey"])
             account_info = AccountInfo._from_response_values(account_data["account"], address)
-            account = Account.parse(account_info, group)
+            account = Account.parse(account_info, group, cache)
             accounts += [account]
         return accounts
 
@@ -320,12 +337,26 @@ class Account(AddressableAccount):
 
         return accounts[0]
 
+    def slot_by_instrument_or_none(self, instrument: Instrument) -> typing.Optional[AccountSlot]:
+        for slot in self.slots:
+            if slot.base_instrument == instrument:
+                return slot
+
+        return None
+
+    def slot_by_instrument(self, instrument: Instrument) -> AccountSlot:
+        slot: typing.Optional[AccountSlot] = self.slot_by_instrument_or_none(instrument)
+        if slot is not None:
+            return slot
+
+        raise Exception(f"Could not find token {instrument} in account {self.address}")
+
     def load_all_spot_open_orders(self, context: Context) -> typing.Dict[str, OpenOrders]:
         spot_open_orders_account_infos = AccountInfo.load_multiple(context, self.spot_open_orders)
         spot_open_orders_account_infos_by_address = {
             str(account_info.address): account_info for account_info in spot_open_orders_account_infos}
         spot_open_orders: typing.Dict[str, OpenOrders] = {}
-        for slot in self.slots:
+        for slot in self.base_slots:
             if slot.spot_open_orders is not None:
                 account_info = spot_open_orders_account_infos_by_address[str(slot.spot_open_orders)]
                 oo = OpenOrders.parse(account_info, slot.base_instrument.decimals,
@@ -342,10 +373,10 @@ class Account(AddressableAccount):
     def __str__(self) -> str:
         info = f"'{self.info}'" if self.info else "(𝑢𝑛-𝑛𝑎𝑚𝑒𝑑)"
         shared_quote: str = f"{self.shared_quote}".replace("\n", "\n        ")
-        slot_count = len(self.slots)
-        slots = "\n        ".join([f"{item}".replace("\n", "\n        ") for item in self.slots])
+        slot_count = len(self.base_slots)
+        slots = "\n        ".join([f"{item}".replace("\n", "\n        ") for item in self.base_slots])
 
-        symbols: typing.Sequence[str] = [slot.base_instrument.symbol for slot in self.slots]
+        symbols: typing.Sequence[str] = [slot.base_instrument.symbol for slot in self.base_slots]
         in_margin_basket = ", ".join(symbols) or "None"
         return f"""« 𝙰𝚌𝚌𝚘𝚞𝚗𝚝 {info}, {self.version} [{self.address}]
     {self.meta_data}
